/* 
 *  $Id: NodeAddress.Java,v 1.3 1999/09/07 21:15:56 tullmann Exp $
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */
package JANOS_ANTS_ANTS_PKG;

import JANOS_CORE_PKG.Address;

/**
 * Routines for manipulating node addresses.  A node address is 
 * simply a 32 bit integer for efficiency, but these convenience
 * routines exist to convert to and from printed representations
 * and so on.
 * <p>
 * This is just here (in Janos) for compatiblity.  All the functions
 * are in the core.
 *
 * @author David Wetherall
 */
public class NodeAddress {

	/**
	 * Insubstantiable
	 */
	private NodeAddress()
	{
	}
  
	/**
	 * Determine if the given integer address is a multicast address.
	 */
	public final static boolean multicast(int a) {
		// XXX move into Address
		return ((a & 0xF0000000) == 0xE0000000);
	}
  
	/**
	 * Generate a string representation of the given address.
	 * Generates standard dotted-quad notation.
	 */
	public final static String toString(int a) {
		return Address.toString(a);
	}
  
	/**
	 * Generate a 32-bit integer from the given string address.
	 * The string must be a dotted-quad.
	 *
	 * @exception Exception if there aren't 4 parts, any of the 4 parts don't parse,
	 * or are out of range.
	 */
	public static int fromString(String name)
		throws Exception //  XXX
	{
		return Address.fromString(name);
	}
}
