/* 
 *  $Id: ClassCode.Java,v 1.3 1999/09/07 21:16:58 tullmann Exp $
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 * 
 */
package JANOS_ANTS_UTILS_PKG;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;

/**
 * Obtain classfile as an array of bytes for a given class name.
 * <p>
 * Superseded by <a href="ClassPathReader.html">ClassPathReader</a>.
 *
 * @author David Wetherall
 */

public class ClassCode {

  /**
   * Convert a fully qualified class name into an array
   * of its bytes that is the classfile. Search the standard
   * classpath and load the file -- can't find any other way
   *
   * @param cn The name of the class to find.  Use '.' for package
   * separator, and do not append '.class'.  
   * @return an array of bytes with the appropriate class file.
   * @todo Does not work for zip archives
   */
  public static byte[] get(String cn) 
    throws Exception 
  {
    // Add a terminating ':' on to the path so that we 
    // are sure we search the last element
    String cp = System.getProperty("java.class.path") + File.pathSeparator;
    String ce = File.separator + cn.replace('.', File.separatorChar) + ".class";
    FileInputStream cfi = null;
    File cf = null;
    int last = 0;
    int pos;
    

    while ((pos = cp.indexOf(File.pathSeparatorChar, last)) > 0) {
      cf = new File(cp.substring(last, pos) + ce);
      last = pos+1;
      try {
	cfi = new FileInputStream(cf);
	break;
      } catch (FileNotFoundException e) {
	continue;
      }
    }
    
    if (cf == null || cfi == null)
      throw new Exception("code not found for "+ cn);
    
    int len = (int)cf.length();
    byte[] cb = new byte[len];

    if (cfi.read(cb) != len)
      throw new Exception("short read for "+ cn);

    return cb;
  }  
}
