/*
 * CommQueue.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_CORE_PKG;

import java.util.Vector; // XXX

/**
 * A shareable FIFO queue of CommSpaceElements.
 * <p>
 * XXX The underlying Vector object is shared, and potentially
 * re-allocated by whomever does an add.  Replace with a
 * linked list of CommSpaceElements.
 *
 * %AUTHOR_CKH%
 */
public final class CommQueue
{
	// Sorry about the inefficient implementation:
	private final Vector elements = new Vector();

	/**
	 * XXX implicit allocation in elements.addElement().
	 */
	public void enqueue(CommSpaceElement elem)
	{
		synchronized(elements)
		{
			elements.addElement(elem);
			elements.notify();
		}
	}

	/**
	 * XXX is it necessary to copy the CommSpaceElement out
	 * of the CommSpace??
	 */
	private static CommSpaceElement copy(CommSpaceElement e)
	{
		return (e == null)?(null):(new CommSpaceElement(e));
	}


	/**
	 * XXX implicit allocation in elements.removeElementAt()
	 * (downsizing).
	 * <p>
	 * May throw InterruptedException if the wait() is
	 * interrupted.  The caller will have to restart the
	 * waitToTake().
	 *
	 * @allocation: creates a copy of the element it returns.
	 */
	public CommSpaceElement waitToTake()
		throws InterruptedException
	{
		synchronized(elements)
		{
			if (elements.size() == 0)
				elements.wait();
			CommSpaceElement elem = (CommSpaceElement)
				elements.firstElement();
			elements.removeElementAt(0);

			return copy(elem);
		}
	}
}
