/*
 * FlowInitializer.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_CORE_PKG;

import java.lang.reflect.Constructor;

/**
 * A stupid class that only exists because
 * java.lang.Runnable specifies that a run
 * method which is public.  Gak.
 */
PACKAGE class FlowInitializer
	implements Runnable
{
	private final Flow flow;
	private final ThreadPool threadPool;
	private final FlowStartSpec flowStartSpec;

	PACKAGE FlowInitializer(Flow flow,
				ThreadPool tp,
				FlowStartSpec fss)
	{
		this.flow = flow;
		this.threadPool = tp;
		this.flowStartSpec = fss;
	}

	public void run()
	{
		try
		{
			FlowClassLoader loader;
			Class c;
			String[] args = null;
			CommSpaceHandle csh = null;

			loader = new FlowClassLoader(flowStartSpec.classSpecs);
			flow.loader = loader;
			c = loader.loadClass(flowStartSpec.startClass, true);

			// This isn't really necessary:
			if(!FlowRun.class.isAssignableFrom(c))
				throw new ClassNotFoundException(
					flowStartSpec.startClass
					+ " is not a subclass of FlowRun");

//			This seemed so easy:
//			  flow.flowRun = (FlowRun) c.newInstance();
//			But no, it's not, because we don't want to require
//			c to have a public constructor:

			// Need to have a VM method that will execute a private/package
			// constructor on the class.  We trust the EE runtime to
			// only pass safe classes.


//System.out.println("A!");
			try
			{
// In Kaffe, getConstructor crashes, so:
//if(true) throw new NoSuchMethodException("kaffe workaround");
				Constructor con = c.getConstructor(new Class[] 
					{ ClassInstantiationKey.class });
				flow.flowRun = (FlowRun) con.newInstance(
					new Object[] {
						new ClassInstantiationKey(c) });
			}
			catch(NoSuchMethodException e)
			{
//System.out.println("B!");
				flow.flowRun = (FlowRun) c.newInstance();
			}
//System.out.println("C!");

			if (flowStartSpec.args != null)
			{
				args = new String[flowStartSpec.args.length];
				for(int i = 0; i < args.length; i++)
				{
					args[i] = new String(flowStartSpec.args[i]);
				}
			}

			if (flowStartSpec.commSpaceHandle != null)
				csh = new CommSpaceHandle(flowStartSpec.commSpaceHandle);

			flow.flowRun.run(flow,
					 threadPool,
					 args,
					 csh); 
		}
		catch(java.lang.reflect.InvocationTargetException e)
		{
			Throwable t = e.getTargetException();
			// XXX handle this better
			DEBUGONLY(Node.println("Child flow "
					       +flow.getName()+ " threw: "));
			t.printStackTrace();
		}
		catch (Throwable t)
		{
			// XXX handle this better
			DEBUGONLY(Node.println("Child flow "
					       +flow.getName()+ " threw: "));
			t.printStackTrace();
		}
	}
}
