/*
 * WriteableRouteTable.Java
 *
 * Copyright (c) 1999 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Janos.
 * 
 * Janos is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 * 
 * Janos is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details, in the file
 * COPYING.  If that file is not present, write to the Free
 * Software Foundation, 59 Temple Place - Suite 330, Boston, MA  
 * 02111-1307, USA
 * 
 * The University of Utah and the Flux Group requests users to
 * return to flux-dist@cs.utah.edu any improvements that they make.
 */
package JANOS_ROUTE_PKG;

import JANOS_CORE_PKG.NoSuchNodeAddress;
import JANOS_CORE_PKG.AddressFormatException;

import java.util.Enumeration; // XXX

/**
 * Interface for an updatable routing table.  The regular
 * route table is read-only.  Access to this class is controlled
 * (or, more acurately, supposed to be controlled) by controls on
 * a flow's namespace.
 * <p>
 * NOTE: If you add/change any interface in here, be sure to
 * update the ANTS DynamicRouteTable as it has to hide all of
 * these interfaces.
 *
 * @see RouteTable
 * %AUTHOR_PAT%
 */
public class WriteableRouteTable
	extends RouteTable
{
	/**
	 * Create a WriteableRouteTable.
	 */
	public WriteableRouteTable() 
	{
		super();
	}

	/**
	 * Get the underlying read-only table.
	 */
	public RouteTable getRouteTable()
	{
		return this;
	}

	public Enumeration getRoutes()
	{
		return super.getRoutes();
	}

	/**
	 * Retrive the current route to get to the given active address.
	 */
	public RouteEntry getRoute(int activeDest)
	{
		return super.getRoute(activeDest);
	}

	public RouteEntry getRoute(Integer activeDest)
	{
		return super.getRoute(activeDest);
	}

	/**
	 * Remove a route from the table.
	 */
	public void removeRoute(RouteEntry r)
	{
		super.package_removeRoute(r);
	}

	/**
	 * Add a route to the table.
	 */
	public void addRoute(RouteEntry route) 
	{
		super.package_addRoute(route);
	}

	/**
	 * Add a route to the table.
	 */
	public void addRoute(int activeDest, int activeNext, String physNext)
		throws NoSuchNodeAddress, AddressFormatException
	{
		super.package_addRoute(new RouteEntry(activeDest, activeNext, physNext));
	}
}

